@@ -8,7 +8,14 @@ from datetime import datetime
 from typing import Any, Optional
 
 from supermodels.exceptions import ValidationError
-from supermodels.fields import timestamp_field, version_field
+from supermodels.fields import (
+    FieldTypeChecker,
+    ImmutableDict,
+    ImmutableList,
+    ImmutableSet,
+    timestamp_field,
+    version_field,
+)
 from supermodels.serdes import (
     DataModelDeserializer,
     DataModelSerializer,
@@ -19,26 +26,43 @@ from supermodels.util import get_class_members, get_timestamp, is_supermodel
 
 def __post_init__(self):
     """Validates the supermodel instance data against its rules."""
-    fails = []
+
+    def validate_rules():
+        fails = []
+
+        for own_field in fields(self):
+            value = getattr(self, own_field.name)
+            rules = own_field.metadata.get("rules", None)
+
+            if rules:
+                rule_fails = rules.validate(value)
+                fails.extend(
+                    [
+                        f"{own_field.name}_{rule_name}_{rule_value}"
+                        for rule_name, rule_value in rule_fails
+                    ]
+                )
+
+        if not fails:
+            return
+
+        msg = "Failing rules: " + "; ".join(fails)
+        self.raise_validation_error(msg)
+
+    validate_rules()
+
+    mapper = {
+        FieldTypeChecker.is_list: ImmutableList,
+        FieldTypeChecker.is_set: ImmutableSet,
+        FieldTypeChecker.is_dict: ImmutableDict,
+    }
 
     for own_field in fields(self):
-        value = getattr(self, own_field.name)
-        rules = own_field.metadata.get("rules", None)
-
-        if rules:
-            rule_fails = rules.validate(value)
-            fails.extend(
-                [
-                    f"{own_field.name}_{rule_name}_{rule_value}"
-                    for rule_name, rule_value in rule_fails
-                ]
-            )
-
-    if not fails:
-        return
-
-    msg = "Failing rules: " + "; ".join(fails)
-    self.raise_validation_error(msg)
+        for type_checker, wrapper in mapper.items():
+            if type_checker(own_field.type):
+                value = getattr(self, own_field.name)
+                object.__setattr__(self, own_field.name, wrapper(value, self.__class__))
+                break
 
 
 def __help__(cls):
